var vows = require("vows");

// Topic object. This is an abstraction around Vows topics.
// Created for ease of extension development
var Topic = {
	// Sets the value of the topic
	"set": function(topic) {
		this._val = topic;
		return this;
	},
	// returns the parent
	"parent": function() {
		return this._parent;
	},
	// returns the raw value
	"end": function() {
		return this._val;
	}
}

// Vow object. This is an abstraction around Vows vows.
// Created for ease of extension development
var Vow = {
	// Sets the value of the vow
	"set": function(vow) {
		this._val = vow;
		return this;
	},
	// returns the parent
	"parent": function() {
		return this._parent;
	},
	// returns the raw value
	"end": function() {
		return this._val;
	}
}

// Context object. This is an abstraction around Vows contexts
var Context = {
	// initialize the context
	"start": function _start() {
		this._val = {};
		return this;
	},
	// end the context. This returns the Object that vow expects to be 
	// a context. This will call end recursively if it exists
	"end": function _end() {
		var ctx = {};
		Object.keys(this._val).forEach((function(key) {
			var val = this._val[key];
			if (val.end) {
				ctx[key] = val.end();
			} else {
				ctx[key] = val;
			}
		}).bind(this));
		return ctx;
	},
	// Sets the topic of the context to be `topic`
	// Or returns a Topic if no `topic` was passed.
	"topic": function _topic(topic) {
		if (topic) {
			this._val.topic = topic;
			return this;
		} else {
			var t = Object.create(Topic);
			t._parent = this;
			this._val.topic = t;
			return t;	
		}
	},
	// Adds a vow to the contex
	// with the name `text` and value `vow`
	// or returns a Vow if no `vow` was passed
	"vow": function _vow(text, vow) {
		if (vow) {
			this._val[text] = vow;	
			return this;
		} else {
			var v = Object.create(Vow);
			v._parent = this;
			this._val[text] = v;
			return v;
		}
	},
	// Create a new sub context and add it to this context
	// with the name `text`. Returns the sub context
	"context": function _context(text) {
		var c = createContext();
		c._parent = this;
		c._batch = this._batch;
		this._val[text] = c;
		return c;
	},
	// Returns the parent context
	"parent": function _parent() {
		return this._parent;
	},
	// returns the batch this context belongs to
	"batch": function _batch() {
		return this._batch;
	},
	// returns the suite this context belongs to
	"suite": function _suite() {
		return this._batch._suite;
	}
};

// A representation of a vows batch.
// A vows batch just contains a bunch of contexts
var Batch = {
	// initialize the object
	"start": function _start() {
		this._val = {};
		return this;
	},
	// end the batch. This transforms it into a vows compliant
	// data structure.
	"end": function _end(){
		var batch = {};
		Object.keys(this._val).forEach((function(ctx) {
			batch[ctx] = this._val[ctx].end();
		}).bind(this));
		return batch;
	},
	// create a new context and adds it to the batch
	// with the name `title`. Returns the context. 
	"context": function _context(title) {
		var c = createContext();
		c._batch = c._parent = this;
		this._val[title] = c;
		return c;
	},
	// returns the suite this batch belongs to.
	"suite": function() {
		return this._suite;
	}
};

// A representation of a vows Suite. Every application
// should onhly have one suite.
var Suite = {
	// initialize the suite. Set's the describtion
	// of the suite to be `name`.
	"start": function _start(name) {
		this._val = [];
		this._suite = vows.describe(name);
		return this;
	},
	// Creates a batch and add's it to the suite.
	// Returns the batch
	"batch": function _batch() {
		var b = createBatch();
		b._suite = this;
		this._val.push(b);
		return b;
	},
	// end the suite and call export on the vows suite
	"export": function _export(m, options) {
		var s = this.end();
		return s.export(m, options);
	},
	// A macro for JSLint
	"exportTo": function _exportTo(m, options) {
		return this.export(m, options);
	},
	// end the suite and call run on the vows suite
	"run": function _run(options, callback) {
		var s = this.end();
		return s.run(options, callback);	
	},
	// end the suite returning a vows Suite.
	"end": function _end() {
		this._val.forEach((function _addBatches(batch) {
			this._suite.addBatch(batch.end());
		}).bind(this));
		return this._suite;
	}
}

// creates a batch by cloning `Batch` and 
// starting it.
var createBatch = function() {
	return Object.create(Batch).start();
}

// Create a context by cloning `Context` and 
// starting it.
var createContext = function() {
	return Object.create(Context).start();
}

// Create a Suite by cloning `Suite` and starting
// it with `name`
var createSuite = function(name) {
	return Object.create(Suite).start(name);
}

// expose objects
exports.Vow = Vow;
exports.Topic = Topic
exports.Context = Context;
exports.Batch = Batch;
exports.Suite = Suite;
exports.context = exports.createContext = createContext;
exports.batch = exports.createBatch = createBatch;
exports.suite = exports.createSuite = createSuite;